//---------------------------------------------------------------------------------
/// \file	MyFunction.cpp
/// \author	Rob Bateman [robthebloke@hotmail.com]
/// \date	2-7-2004
/// \brief	This is the simplest possible example of how to write a mel command
///			that can be added into Maya via a plugin. You can call the command 
///			from mel by typing, "SimpleCommand". This simply executes a mel call 
///			via MGlobal::executeCommand to create a sphere.
///	
//---------------------------------------------------------------------------------

#ifdef WIN32
	#define NT_PLUGIN
#endif

// include any maya headers we need.
#include "MyFunction.h"
#include <maya/MSelectionList.h>
#include <maya/MArgList.h>


//---------------------------------------------------------------------------------
bool MyFunction::isUndoable() const {
	return false;
}

//---------------------------------------------------------------------------------
/// \brief	this function is called by maya to return a new instance of our mel function
/// \return	the new mel command object
///
void* MyFunction::creator() {
	return new MyFunction;
}

//---------------------------------------------------------------------------------
/// \brief	an internal function to simply add a set of lights to the mesh object
/// 		specified.
/// \param	oMesh	-	the handle to the object to attach the lights to
/// \return	a status code
/// 
MStatus MyFunction::addLightsToMesh(MObject oMesh) {

	MFnPointLight fnLight;
	// attach a function set to the mesh
	MFnMesh fnMesh( oMesh );

	// get the parent transform of the mesh and attach a function set
	MObject oParent = fnMesh.parent(0);
	MFnTransform fnParent( oParent );


	// retrieve the mesh vertex data
	MPointArray points;
	fnMesh.getPoints( points );


	// loop through each point on the mesh
	for(unsigned int i=0; i<points.length(); ++i )
	{
		// create a vector from the current mesh vertex
		MVector V;
		V.x = points[i].x;
		V.y = points[i].y;
		V.z = points[i].z;

		MFnTransform fnX;

		// create a transform for the light instance, specifying the meshes
		// parent transform as the parent for the new transform. Basically
		// we are going to add the light instance as a child of the mesh. This
		// way when we move the mesh, all of the lights will follow...
		//
		fnX.create( fnMesh.parent(0) );

		// set the translation value
		fnX.setTranslation( V, MSpace::kTransform );

		// create a point light and store the parent transform of the light
		MObject oLight = fnLight.create(fnX.object());

		MColor col(1,0,0);
		fnLight.setColor(col);

		// add the light to the transform we created for it. If the last paramater
		// was false, then it would unparent the light from any
		fnX.addChild( oLight , MFnDagNode::kNextPos , true );
	}
	return MS::kSuccess;
}

//---------------------------------------------------------------------------------
/// \brief	This function impliments our very first simple command plugin.
/// \param	args - the argument list that was passes to the command from MEL
/// \return	MS::kSuccess - command succeeded
///			MS::kFailure - command failed (returning this value will cause the
///         	           MEL script that is being run to terminate unless the
///         	           error is caught using a "catch" statement.
///
MStatus MyFunction::doIt( const MArgList& args )

{
	// the return status
	MStatus stat = MS::kSuccess;

	// we need to use a light function set to do anything useful ;)
	MFnPointLight fnLight;

	// will hold the flag for doing only selected items
	bool doSelected=false;

	// loop through each argument in the arg list
	for(unsigned int i=0;i<args.length();++i)
	{
		// if we read -sl or -selected, check the selected flag
		if( args.asString(i) == "-sl" || args.asString(i) == "-selected" )
		{
			// make sure we have enough arguments left in the arg list
			if(++i == args.length()) {
				// display error if it fucked up
				MGlobal::displayError("Missing argument for -selected/-sl flag");
				return MS::kFailure;
			}

			// read the next argument back as a float
			doSelected = args.asBool(i);
		}
	}

	// if we are only handling selected items
	if(doSelected)
	{
		// get the current selection list from maya
		MSelectionList selected;
		MGlobal::getActiveSelectionList(selected);

		// loop through the selected items
		for(unsigned int i=0;i<selected.length();++i)
		{
			// get the MObject for this item
			MObject oObj;
			selected.getDependNode(i,oObj);

			// if we have a mesh selected
			if(oObj.apiType()==MFn::kMesh)
				addLightsToMesh( oObj );
			else
			// if we have a transform of some sort
			if( oObj.hasFn(MFn::kTransform) )
			{
				// check all of it's children to see if a mesh is parented directly underneath
				MFnTransform fn(oObj);
				for(unsigned int j=0;j<fn.childCount();++j)
				{
					// check to see if this child is a mesh
					MObject oChild = fn.child(j);
					if(oChild.apiType()==MFn::kMesh)
						addLightsToMesh( oChild );

				}
			}
		}
	}
	else
	{
		// iterate through all of the meshes within the scene
		MItDependencyNodes it(MFn::kMesh);
		while( !it.isDone() )
		{
			// get the MObject handle to the mesh
			MObject oMesh = it.item();

			// create lights on the mesh
			addLightsToMesh( it.item() );

			it.next();
		}
	}

	return stat;
}

